#!/usr/bin/python

# b3m - BlackBox Background Menu generator
# given a directory, generate the proper bb menu

# Author: Daniel Lyons <fusion+b3m@storytotell.org>
# $Revision: 1.2 $

# RCS info: $Id: b3m.py,v 1.2 2004/06/20 18:35:56 fusion Exp fusion $

import os
import sys

# returns a case-insensitive sorted copy of the list
def insensitive_sort(l):
	# copy l
	q = l[:]

	# the insensitive compare function: upper case both words and then compare
	def i_cmp(first, second):
		return cmp(first.upper(), second.upper())

	# sort our copy
	q.sort(i_cmp)
	
	# return it
	return q

# given a name of the form "this is my name", return it capitalized
# like so: "This Is My Name"
def properize(name):
	return " ".join([ word.capitalize() for word in name.split(' ')])

# base class for both Directory and Image
class PathItem(object):
	def __init__(self, path):
		self.path = path
		
	def get_name(self):
		return properize(os.path.basename(os.path.splitext(self.path)[0]))

	name = property(get_name)	

# hold the path of a directory and the contents
class Directory(PathItem):
	def __init__(self, path):
		PathItem.__init__(self, path)
		self.contents = []
		self.load_path()

	def load_path(self):
		# get a sorted list of files
		files = insensitive_sort(os.listdir(self.path))

		# for each file...
		for file in files:
			# get the path to that file in fpath
			fpath = os.path.join(self.path, file)

			# all files are assumed to be images.  this could be tightened up
			if os.path.isfile( fpath ):
				self.contents.append( Image(fpath) )

			# recur into directories
			# we do not recur into hidden directories (starting with '.')
			elif os.path.isdir( fpath ) and file[0] != '.':
				self.contents.append( Directory(fpath) )

	def __repr__(self):
		return repr(self.contents)

	# boilerplate visitor code
	def visit(self, visitor):
		visitor.visit_dir(self)

# container for other types of files (we assume everything not a directory is an image)
class Image(PathItem):
	def __init__(self, path):
		PathItem.__init__(self, path)

	def __repr__(self):
		return "<Image: '%s'>" % self.name

	# boilerplate visitor code
	def visit(self, visitor):
		visitor.visit_image(self)

def parse_dir(path):
	dir = Directory(path)

# generates a nice pretty menu for a directory
# uses the Visitor pattern for implementing the recursion
class BBMenuGen:
	def __init__(self, dir):
		self.menu = ""
		self.il = 0
		dir.visit(self)

	def indent(self):
		self.il += 2

	def dedent(self):
		self.il -= 2

	def visit_dir(self, dir):
		# start the menu
		self.menu += self.il * ' ' + '[submenu] (%s)\n' % dir.name

		self.indent() # indent
		for file in dir.contents:
			file.visit(self)  # visit all the contents of this directory
		self.dedent() # unindent

		# end the menu
		self.menu += self.il * ' ' + '[end]\n'

	def visit_image(self, file):
		# generate the exec line for an image
		self.menu += self.il * ' ' + \
					 '[exec] (%s) {Esetroot -f \'%s\'}\n' % \
					 (file.name, file.path)

# this line does all the work
if __name__ == "__main__":
	if len(sys.argv) < 2:
		print "Usage: %s <directory of images>"
	else:
		print BBMenuGen(Directory(sys.argv[1])).menu
